import logging
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class WebScrapeInput(BaseModel):
    url: str = Field(description="URL of web page to be scraped and ingested into knowledge base")

class WebScrapeTool(BaseTool):
    name: str = "Graphlit web scrape tool"
    description: str = """Scrapes web page into knowledge base.
    Returns Markdown text and metadata extracted from web page."""
    args_schema: Type[BaseModel] = WebScrapeInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the WebScrapeTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting web pages. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, url: str) -> Optional[str]:
        content_id = None

        try:
            response = await self.graphlit.client.ingest_uri(
                uri=url,
                workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                is_synchronous=True,
                correlation_id=self.correlation_id
            )

            content_id = response.ingest_uri.id if response.ingest_uri is not None else None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        if content_id is None:
            return None

        try:
            response = await self.graphlit.client.get_content(
                id=content_id
            )

            if response.content is None:
                return None

            logger.debug(f'WebScrapeTool: Retrieved content by ID [{content_id}].')

            results = helpers.format_content(response.content)

            text = "\n".join(results)

            return text
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

    def _run(self, url: str) -> Optional[str]:
        return helpers.run_async(self._arun, url)



import logging
import json
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class ExtractTextInput(BaseModel):
    text: str = Field(description="Text to be extracted with LLM.")
    model: BaseModel = Field(description="Pydantic model which describes the data which will be extracted")
    prompt: Optional[str] = Field(description="Text prompt which is provided to LLM to guide data extraction, optional.", default=None)

class ExtractTextTool(BaseTool):
    name: str = "Graphlit JSON text data extraction tool"
    description: str = """Extracts JSON data from text using LLM.
    Returns extracted JSON from text."""
    args_schema: Type[BaseModel] = ExtractTextInput

    graphlit: Graphlit = Field(None, exclude=True)

    specification_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, specification_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the ExtractTextTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            specification_id (Optional[str]): ID for the LLM specification to use. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.specification_id = specification_id
        self.correlation_id = correlation_id

    async def _arun(self, text: str, model: BaseModel, prompt: Optional[str] = None) -> Optional[str]:
        default_prompt = """
        Extract data using the tools provided.
        """

        try:
            response = await self.graphlit.client.extract_text(
                specification=input_types.EntityReferenceInput(id=self.specification_id) if self.specification_id is not None else None,
                tools=[input_types.ToolDefinitionInput(name=model.__name__, schema=model.model_dump_json())],
                prompt=default_prompt if prompt is None else prompt,
                text=text,
                correlation_id=self.correlation_id
            )

            if response.extract_text is None:
                logger.debug('Failed to extract text.')
                return None

            extractions = response.extract_text

            json_array = json.loads('[' + ','.join(extraction.value for extraction in extractions) + ']')

            return json.dumps(json_array, indent=4)
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            print(str(e))
            raise ToolException(str(e)) from e
        except Exception as e:
            logger.error(str(e))
            print(str(e))
            raise ToolException(str(e)) from e

    def _run(self, text: str, model: BaseModel, prompt: Optional[str] = None) -> Optional[str]:
        return helpers.run_async(self._arun, text, model, prompt)



import logging
import time
import os
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class GoogleEmailIngestInput(BaseModel):
    search: Optional[str] = Field(description="Text to search for within ingested emails.", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of emails from Google Email account to be read.", default=10)

class GoogleEmailIngestTool(BaseTool):
    name: str = "Graphlit Google Email ingest tool"
    description: str = """Ingests emails from Google Email account into knowledge base.
    Returns extracted Markdown text and metadata from emails."""
    args_schema: Type[BaseModel] = GoogleEmailIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the GmailIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting emails. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        refresh_token = os.environ['GOOGLE_EMAIL_REFRESH_TOKEN']

        if refresh_token is None:
            raise ToolException('Invalid Google Email refresh token. Need to assign GOOGLE_EMAIL_REFRESH_TOKEN environment variable.')

        client_id = os.environ['GOOGLE_EMAIL_CLIENT_ID']

        if client_id is None:
            raise ToolException('Invalid Google Email client identifier. Need to assign GOOGLE_EMAIL_CLIENT_ID environment variable.')

        client_secret = os.environ['GOOGLE_EMAIL_CLIENT_SECRET']

        if client_secret is None:
            raise ToolException('Invalid Google Email client secret. Need to assign GOOGLE_EMAIL_CLIENT_SECRET environment variable.')

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name='Google Email',
                    type=enums.FeedTypes.EMAIL,
                    email=input_types.EmailFeedPropertiesInput(
                        type=enums.FeedServiceTypes.GOOGLE_EMAIL,
                        google=input_types.GoogleEmailFeedPropertiesInput(
                            type=enums.EmailListingTypes.PAST,
                            refreshToken=refresh_token,
                            clientId=client_id,
                            clientSecret=client_secret,
                        ),
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                return None

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, search: Optional[str] = None, read_limit: Optional[int] = None) -> str:
        return helpers.run_async(self._arun, search, read_limit)



class ToolException(Exception):
    """
    Custom exception class for tool-related errors.

    Args:
        error_message (str): A descriptive error message for the exception.
    """
    def __init__(self, error_message: str):
        super().__init__(error_message)
        self.error_message = error_message

    def __str__(self) -> str:
        """
        String representation of the exception.

        Returns:
            str: The error message.
        """
        return self.error_message



from .base_tool import BaseTool
from .crewai_converter import CrewAIConverter
from .exceptions import ToolException
from .retrieval.content_retrieval_tool import ContentRetrievalTool
from .retrieval.person_retrieval_tool import PersonRetrievalTool
from .retrieval.organization_retrieval_tool import OrganizationRetrievalTool
from .extraction.extract_text_tool import ExtractTextTool
from .extraction.extract_url_tool import ExtractURLTool
from .extraction.extract_web_page_tool import ExtractWebPageTool
from .generation.prompt_tool import PromptTool, PromptToolInput
from .generation.describe_image_tool import DescribeImageTool
from .generation.describe_web_page_tool import DescribeWebPageTool
from .generation.generate_summary_tool import GenerateSummaryTool
from .generation.generate_bullets_tool import GenerateBulletsTool
from .generation.generate_headlines_tool import GenerateHeadlinesTool
from .generation.generate_social_media_posts_tool import GenerateSocialMediaPostsTool
from .generation.generate_questions_tool import GenerateQuestionsTool
from .generation.generate_keywords_tool import GenerateKeywordsTool
from .generation.generate_chapters_tool import GenerateChaptersTool
from .ingestion.url_ingest_tool import URLIngestTool
from .ingestion.local_ingest_tool import LocalIngestTool
from .ingestion.web_scrape_tool import WebScrapeTool
from .ingestion.web_crawl_tool import WebCrawlTool
from .ingestion.web_search_tool import WebSearchTool
from .ingestion.web_map_tool import WebMapTool
from .ingestion.reddit_ingest_tool import RedditIngestTool
from .ingestion.notion_ingest_tool import NotionIngestTool
from .ingestion.microsoft_email_ingest_tool import MicrosoftEmailIngestTool
from .ingestion.google_email_ingest_tool import GoogleEmailIngestTool
from .ingestion.github_issue_ingest_tool import GitHubIssueIngestTool
from .ingestion.jira_issue_ingest_tool import JiraIssueIngestTool
from .ingestion.linear_issue_ingest_tool import LinearIssueIngestTool
from .ingestion.microsoft_teams_ingest_tool import MicrosoftTeamsIngestTool
from .ingestion.discord_ingest_tool import DiscordIngestTool
from .ingestion.slack_ingest_tool import SlackIngestTool
from .ingestion.rss_ingest_tool import RSSIngestTool



from typing import Any, cast

try:
    from crewai_tools.tools.base_tool import BaseTool as CrewAIBaseTool
except ImportError as e:
    raise ImportError(
        "CrewAIConverter requires the `crewai_tools` package. "
        "Install it using `pip install graphlit-tools[crewai]`."
    ) from e

from .base_tool import BaseTool

class CrewAIConverter(CrewAIBaseTool):
    """Tool to convert Graphlit tools into CrewAI tools."""

    graphlit_tool: Any

    def _run(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        tool = cast(BaseTool, self.graphlit_tool)

        return tool.run(*args, **kwargs)

    async def _arun(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        tool = cast(BaseTool, self.graphlit_tool)

        return await tool.arun(*args, **kwargs)

    @classmethod
    def from_tool(cls, tool: Any, **kwargs: Any) -> "CrewAIConverter":
        if not isinstance(tool, BaseTool):
            raise ValueError(f"Expected a Graphlit tool, got {type(tool)}")

        tool = cast(BaseTool, tool)

        if tool.args_schema is None:
            raise ValueError("Invalid arguments JSON schema.")

        return cls(
            name=tool.name,
            description=tool.description,
            args_schema=tool.args_schema,
            graphlit_tool=tool,
            **kwargs,
        )
